"""The module that defines the ``BaseDirectory`` model.

SPDX-License-Identifier: AGPL-3.0-only OR BSD-3-Clause-Clear
"""

import typing as t
from dataclasses import dataclass, field

import cg_request_args as rqa

from ..utils import to_dict


@dataclass
class BaseDirectory:
    """The base JSON representation of a directory."""

    #: This is a directory
    type: "t.Literal['directory']"
    #: The id of the directory, this can be used to retrieve it later on.
    id: "str"
    #: The name of the directory, this does not include the name of any
    #: parents.
    name: "str"

    raw_data: t.Optional[t.Dict[str, t.Any]] = field(init=False, repr=False)

    data_parser: t.ClassVar = rqa.Lazy(
        lambda: rqa.FixedMapping(
            rqa.RequiredArgument(
                "type",
                rqa.StringEnum("directory"),
                doc="This is a directory",
            ),
            rqa.RequiredArgument(
                "id",
                rqa.SimpleValue.str,
                doc=(
                    "The id of the directory, this can be used to retrieve it"
                    " later on."
                ),
            ),
            rqa.RequiredArgument(
                "name",
                rqa.SimpleValue.str,
                doc=(
                    "The name of the directory, this does not include the name"
                    " of any parents."
                ),
            ),
        ).use_readable_describe(True)
    )

    def to_dict(self) -> t.Dict[str, t.Any]:
        res: t.Dict[str, t.Any] = {
            "type": to_dict(self.type),
            "id": to_dict(self.id),
            "name": to_dict(self.name),
        }
        return res

    @classmethod
    def from_dict(
        cls: t.Type["BaseDirectory"], d: t.Dict[str, t.Any]
    ) -> "BaseDirectory":
        parsed = cls.data_parser.try_parse(d)

        res = cls(
            type=parsed.type,
            id=parsed.id,
            name=parsed.name,
        )
        res.raw_data = d
        return res
